<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2007 The Cacti Group                                      |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
 | Cacti: The Complete RRDTool-based Graphing Solution                     |
 +-------------------------------------------------------------------------+
 | This code is designed, written, and maintained by the Cacti Group. See  |
 | about.php and/or the AUTHORS file for specific developer information.   |
 +-------------------------------------------------------------------------+
 | http://www.cacti.net/                                                   |
 +-------------------------------------------------------------------------+
*/

# Allow indexing of the script to provide the function name to ConfigStore
if ( (isset($argv[1])) && (trim($argv[1]) === 'index') ) {
    echo "cisco_ios_get_config\n";
    exit;
}

include_once($config['base_path'] . '/plugins/configstore/functions.php');

global $config, $log;

include_once($config['base_path'] . '/plugins/configstore/include/logger.class.php');

$log = new Logging();

function cisco_ios_get_config ($device) {
	global $log, $config;

	$info = plugin_configstore_retrieve_account($device['id']);
	$token = '0';

	if ( ($info['account_type'] == '1') || ($info['account_type'] == '2') || ($info['account_type'] == '3') ) {
		$token = '1';
		$backup_results = cisco_ios_snmp_get_config($device, $info);
		if ($backup_results) {
			return $backup_results;
		} else {
			return false;
		}
	}
        if ($info['account_type'] == '4') {
		$token = '1';
                if(!file_exists($config['base_path'] . '/plugins/configstore/lib/cisco_Catos-telnet.exp')) {
                       return false;
                } else {
			$backup_results = cisco_ios_term_get_config($device, $info);
			if ($backup_results) {
                                return $backup_results;
                        } else {
                                return false;
                        }
                }
        }
	if ($info['account_type'] == '5') {
		$token = '1';
       		if(!file_exists($config['base_path'] . '/plugins/configstore/lib/cisco_Catos-ssh.exp')) {
                       return false;
                } else {
			$backup_results = cisco_ios_term_get_config($device, $info);
			if ($backup_results) {
				return $backup_results;
			} else {
				return false;
			}
                }
	}
	if ($info['backup_protocol'] == '3') {
		$token = '1';
		if(!file_exists($config['base_path'] . '/plugins/configstore/lib/cisco_ios-scpserver.exp')) {
			return false;
		} else {
			$backup_results = cisco_ios_scpserver_get_config($device, $info);
			if ($backup_results) {
				return $backup_results;
			} else {
				return false;
			}
		}
	}

	if ($token == '0') {
		return false;
	}
}

function cisco_ios_snmp_get_config($device, $info) {
	global $log, $config;

	$backup_protocol = '';
        $scp_username = '';
        $scp_password = '';
        $backup_filename = $device['hostname'] . ".cfg";
        $backup_path = $device['backuppath'];
        $backup_server = $device['backup_server'];
        $cmd = '';

        $snmpset = read_config_option('plugin_configstore_snmpset_path');
        if (!is_file($snmpset)) {
                return false;
        }

        if (!isset($info['snmp_version'])) {
                return false;
        }

        if (!isset($info['snmp_retries'])) {
                return false;
        }

        if (!isset($info['snmp_timeout'])) {
                return false;
        }

        if (!isset($info['snmp_port'])) {
                return false;
        }

        if ( ($info['snmp_version'] == '1') || ($info['snmp_version'] == '2c') ) {
                if (!isset($info['snmp_writestring'])) {
                        return false;
                }

		$cmd = "$snmpset " . "-r " . $info['snmp_retries'] . " " . "-t " . $info['snmp_timeout'] . " " . "-Oqv " . "-v " . $info["snmp_version"] . " " . "-c " . $info['snmp_writestring'] . " " . $device['ipaddress'] . ":" . $info['snmp_port'];

        }

	if ($info['snmp_version'] == '3') {
        	if (!isset($info['snmp_username'])) {
                	return false;
        	}

        	if (!isset($info['snmp_password'])) {
                	return false;
        	}

        	if (!isset($info['snmp_auth_protocol'])) {
                	return false;
        	}

        	if (!isset($info['snmp_priv_protocol'])) {
                	return false;
        	}

        	if (!isset($info['snmp_priv_passphrase'])) {
                	return false;
        	}

		$cmd = "$snmpset " . "-r " . $info['snmp_retries'] . " " . "-t " . $info['snmp_timeout'] . " " . "-Oqv " . "-v " . $info["snmp_version"] . " " . "-u " . $info['snmp_username'] . " " . "-a " . $info['snmp_auth_protocol'] . " " . "-A " . $info['snmp_password'] . " " . "-x " . $info['snmp_priv_protocol'] . " " . "-X " . $info['snmp_priv_passphrase'] . " " . "-l authPriv " . $device['ipaddress'] . ":" . $info['snmp_port'];

	}

        $debug = "DEBUG:\r\r";

	if ( (isset($info['backup_protocol'])) && ($info['backup_protocol'] == "1") ) {
                $backup_protocol = "1";
        } elseif ( (isset($info['backup_protocol'])) && ($info['backup_protocol'] == "2") ) {
                $backup_protocol = "4";
                $scp_username = read_config_option('plugin_configstore_scp_username');
                $scp_password = read_config_option('plugin_configstore_scp_password');
	} else {
                return false;
        }

        $snmp_cmds_tftp = array(
                /* The ConfigCopyProtocol is set (TFTP=1 or SCP=4) */
                "$cmd 1.3.6.1.4.1.9.9.96.1.1.1.1.2.111 i" =>  "$backup_protocol",

                /* Set the SourceFileType to running-config */
                "$cmd 1.3.6.1.4.1.9.9.96.1.1.1.1.3.111 i" => "4",

                /* Set the DestinationFileType to networkfile */
                "$cmd 1.3.6.1.4.1.9.9.96.1.1.1.1.4.111 i" => "1",

                /* Sets the ServerAddress to the IP address of the Backup server */
                "$cmd 1.3.6.1.4.1.9.9.96.1.1.1.1.5.111 a" => "$backup_server",

                /* Sets the CopyFilename to your desired file name */
                "$cmd 1.3.6.1.4.1.9.9.96.1.1.1.1.6.111 s" => "$backup_filename",

                /* Sets the CopyStatus to active which starts the copy process */
                "$cmd 1.3.6.1.4.1.9.9.96.1.1.1.1.14.111 i" => "1");

	$snmp_cmds_scp = array(
                /* The ConfigCopyProtocol is set (TFTP=1 or SCP=4) */
                "$cmd 1.3.6.1.4.1.9.9.96.1.1.1.1.2.111 i" =>  "$backup_protocol",

        	/* The username for SCP if ConfigCopyProtocol is SCP */
        	"$cmd 1.3.6.1.4.1.9.9.96.1.1.1.1.7.111 s" => "$scp_username",

        	/* The password for SCP if ConfigCopyProtocol is SCP */
        	"$cmd 1.3.6.1.4.1.9.9.96.1.1.1.1.8.111 s" => "$scp_password",

                /* Set the SourceFileType to running-config */
                "$cmd 1.3.6.1.4.1.9.9.96.1.1.1.1.3.111 i" => "4",

                /* Set the DestinationFileType to networkfile */
                "$cmd 1.3.6.1.4.1.9.9.96.1.1.1.1.4.111 i" => "1",

                /* Sets the ServerAddress to the IP address of the Backup server */
                "$cmd 1.3.6.1.4.1.9.9.96.1.1.1.1.5.111 a" => "$backup_server",

                /* Sets the CopyFilename to your desired file name */
                "$cmd 1.3.6.1.4.1.9.9.96.1.1.1.1.6.111 s" => "$backup_filename",

                /* Sets the CopyStatus to active which starts the copy process */
                "$cmd 1.3.6.1.4.1.9.9.96.1.1.1.1.14.111 i" => "1");

	$snmp_cmds = array();
	if ($backup_protocol == "1") {
		$snmp_cmds = $snmp_cmds_tftp;
	} elseif ($backup_protocol == "4") {
		$snmp_cmds = $snmp_cmds_scp;
	}

	$results = array();
	$success = 1;
	foreach ($snmp_cmds as $snmp_cmd=>$value) {
		$result = array();
		exec("$snmp_cmd $value", $result);
		$result_string = implode('', $result);
		$result_string = trim($result_string, '"');
		if ( (isset($result_string)) && ($result_string == $value) ) {
			$success_string = "SUCCESS";
			if (isset($snmp_cmd)) {
				if ( ($info['snmp_version'] == '1') || ($info['snmp_version'] == '2c') ) {
					$snmp_cmd = str_replace($info['snmp_writestring'], "<rwcommunity>", $snmp_cmd);
				}
				if ($info['snmp_version'] == '3') {
					$snmp_cmd = str_replace($info['snmp_password'], "<snmpAuthpass>", $snmp_cmd);
					$snmp_cmd = str_replace($info['snmp_priv_passphrase'], "<snmpPrivpass>", $snmp_cmd);
				}
			} else {
				$snmp_cmd = '';
			}
			array_push($results, "$snmp_cmd $value $success_string");
		} else {
			$success = 1;
			$success_string = "FAILED";
                        if (isset($snmp_cmd)) {
				if ( ($info['snmp_version'] == '1') || ($info['snmp_version'] == '2c') ) {
                                	$snmp_cmd = str_replace($info['snmp_writestring'], "<rwcommunity>", $snmp_cmd);
				}
				if ($info['snmp_version'] == '3') {
                                        $snmp_cmd = str_replace($info['snmp_password'], "<snmpAuthpass>", $snmp_cmd);
                                        $snmp_cmd = str_replace($info['snmp_priv_passphrase'], "<snmpPrivpass>", $snmp_cmd);
                                }
                        } else {
                                $snmp_cmd = '';
                        }
			array_push($results, "$snmp_cmd  $value  $success_string" . "\r" . $result_string);
			$debug = implode("\r", $results);
			exec("$cmd 1.3.6.1.4.1.9.9.96.1.1.1.1.14.111 i 6", $clear_result);
			$clear_result_string = implode("\r", $clear_result);
			$debug .= "\r$clear_result_string";
			plugin_configstore_save_debug($device, $debug);
			return false;
		}
	}

	$c=0;
	while($c < 5) {
		if (is_file("$backup_path/$backup_filename")) {
			$success = 0;
			/* Sets the CopyStatus to delete which cleans all saved informations out of the MIB */
                	exec("$cmd 1.3.6.1.4.1.9.9.96.1.1.1.1.14.111 i 6", $clear_result);
                	$clear_string = implode('', $clear_result);
			if ($clear_string == '6') {
				$cmd = str_replace($info['snmp_writestring'], "<rwcommunity>", $cmd);
				array_push($results, "$cmd 1.3.6.1.4.1.9.9.96.1.1.1.1.14.111 i 6 SUCCESS");
			} else {
				$cmd = str_replace($info['snmp_writestring'], "<rwcommunity>", $cmd);
                                array_push($results, "$cmd 1.3.6.1.4.1.9.9.96.1.1.1.1.14.111 i 6 FAILED");
			}
			break;
		} else {
			sleep(1);
			$c++;
		}
	}

        $debug = implode("\r", $results);
        if ($success == 0) {
                $backup_results = process_cisco_ios_backup($device, $debug);
                return $backup_results;
        } else {
		plugin_configstore_save_debug($device, $debug);
                return false;
        }
}

function cisco_ios_term_get_config ($device, $info) {
	global $log, $config;

	$backup_protocol = '';
	$scp_username = '';
	$scp_password = '';

	$expect = read_config_option('plugin_configstore_expect_path');
	if (!is_file($expect)) {
		return false;
	}

	if ( (isset($info['backup_protocol'])) && ($info['backup_protocol'] == "1") ) {
		$backup_protocol = "tftp";
	} elseif ( (isset($info['backup_protocol'])) && ($info['backup_protocol'] == "2") ) {
		$backup_protocol = "scp";
		$scp_username = read_config_option('plugin_configstore_scp_username');
                $scp_password = read_config_option('plugin_configstore_scp_password');
	} else {
		return false;
	}

	$enablepw = "0";
	if ( (isset($info['enablepw'])) && (!empty($info['enablepw'])) ) {
		$enablepw = $info['enablepw'];
	}

        $backup_filename = $device['hostname'] . ".cfg";
        $backup_path = $device['backuppath'];
        $backup_server = $device['backup_server'];
	$cmd;

	if ( (isset($info['account_type'])) && ($info['account_type'] == '4') ) {
		$telnet = read_config_option('plugin_configstore_telnet_path');
		if (!is_file($telnet)) {
			return false;
		}

		if ( (isset($backup_protocol)) && ($backup_protocol == "tftp") ) {
			$cmd = "$expect " .  $config['base_path'] . "/plugins/configstore/lib/cisco_Catos-telnet.exp " . $device['ipaddress'] . " " . $info['username'] . " " . $info['password'] . " " . $enablepw . " " .$backup_protocol . " " . $backup_server . " " . $backup_filename . " " . $telnet;
		} elseif ( (isset($backup_protocol)) && ($backup_protocol == "scp") ) {
			$cmd = "$expect " .  $config['base_path'] . "/plugins/configstore/lib/cisco_Catos-telnet.exp " . $device['ipaddress'] . " " . $info['username'] . " " . $info['password'] . " " . $enablepw . " " .$backup_protocol . " " . $backup_server . " " . $backup_filename . " " . $telnet . " " . $scp_username . " " . $scp_password;
		}
	} elseif ( (isset($info['account_type'])) && ($info['account_type'] == '5') ) {
		$ssh = read_config_option('plugin_configstore_ssh_path');
		if (!is_file($ssh)) {
			return false;
		}

		if ( (isset($backup_protocol)) && ($backup_protocol == "tftp") ) {
                        $cmd = "$expect " .  $config['base_path'] . "/plugins/configstore/lib/cisco_Catos-ssh.exp " . $device['ipaddress'] . " " . $info['username'] . " " . $info['password'] . " " . $enablepw . " " .$backup_protocol . " " . $backup_server . " " . $backup_filename . " " . $ssh;
                } elseif ( (isset($backup_protocol)) && ($backup_protocol == "scp") ) {
			$cmd = "$expect " .  $config['base_path'] . "/plugins/configstore/lib/cisco_ios-ssh.exp " . $device['ipaddress'] . " " . $info['username'] . " " . $info['password'] . " " . $enablepw . " " .$backup_protocol . " " . $backup_server . " " . $backup_filename . " " . $ssh . " " . $scp_username . " " . $scp_password;
		}
	} else {
		return false;
	}

	$success;
	$debug = "DEBUG:\r\r";
	exec($cmd, $result, $return_code);

	if ( (!isset($result)) || (sizeof($result) < 1) ) {
    	} else {
		foreach ($result as $r) {
			if (preg_match("/\bRETURN CODE: 0\b/", $r)) {
				$success = 0;
			} elseif (preg_match("/\bRETURN CODE: 1\b/", $r)) {
				$success = 1;
			} elseif (preg_match("/\bRETURN CODE: 2\b/", $r)) {
				$success = 2;
			} else {
				$success = 3;
			}
		}
	}

	$debug = implode("\r", $result);
	if ($success == 0) {
		$backup_results = process_cisco_ios_backup($device, $debug);
		return $backup_results;
	} else {
		plugin_configstore_save_debug($device, $debug);
		return false;
	}
}

function cisco_ios_scpserver_get_config ($device, $info) {
        global $log, $config;

        $cmd = '';

        $expect = read_config_option('plugin_configstore_expect_path');
        if (!is_file($expect)) {
                return false;
        }
        if ( (isset($info['backup_protocol'])) && ($info['backup_protocol'] == "3") ) {
                $scp = read_config_option('plugin_configstore_scp_path');
                if (!is_file($scp)) {
                        return false;
                }
                $backup_filename = $device['hostname'] . ".cfg";
                $backup_path = $config['base_path'] . "/" . $device['backuppath'];
                $scp_username = $info['username'];
                $scp_password = $info['password'];

                if ( (isset($scp_username)) && (isset($scp_password)) && (isset($backup_filename)) && (isset($backup_path)) ) {
                        if (!is_file($config['base_path'] . "/plugins/configstore/lib/cisco_ios-scpserver.exp")) {
                                return false;
                        } else {
                                $cmd = "$expect " .  $config['base_path'] . "/plugins/configstore/lib/cisco_ios-scpserver.exp " . $device['ipaddress'] . " " . $scp_username . " " . $scp_password . " " . $backup_path . " " . $backup_filename . " " . $scp;
                        }
                } else {
                        return false;
                }
        } else {
                return false;
        }
        $success = '';
        $debug = "DEBUG:\r\r";
        exec($cmd, $result, $return_code);

        if ( (!isset($result)) || (sizeof($result) < 1) ) {
        } else {
                foreach ($result as $r) {
                        if (preg_match("/\bRETURN CODE: 0\b/", $r)) {
                                $success = 0;
                        } elseif (preg_match("/\bRETURN CODE: 1\b/", $r)) {
                                $success = 1;
                        } elseif (preg_match("/\bRETURN CODE: 2\b/", $r)) {
                                $success = 2;
                        } else {
                                $success = 3;
                        }
                }
        }

        $debug = implode("\r", $result);
        if ($success == 0) {
                $backup_results = process_cisco_ios_backup($device, $debug);
                return $backup_results;
        } else {
		plugin_configstore_save_debug($device, $debug);
                return false;
        }
}

function process_cisco_ios_backup ($device, $debug) {
	global $log, $config;

	$backup_filename = $device['hostname'] . ".cfg";
	$backup_path = $device['backuppath'];
	$filename = $device['hostname'];
	$dir = $device['directory'];
	$hostname = $device['hostname'];

	$x = 0;
	$data = array();
	while ($x < 30) {
	        if (file_exists("$backup_path/$backup_filename")) {
	        	clearstatcache();
	                $file = fopen("$backup_path/$backup_filename", 'r');
	                if (filesize("$backup_path/$backup_filename") > 0) {
	                	$data = fread($file, filesize("$backup_path/$backup_filename"));
	                }
	                fclose($file);
			break;
	        }
	        sleep(1);
	        $x++;
	}

	if (!file_exists("$backup_path/$backup_filename")) {
		plugin_configstore_save_debug($device, $debug);
	        return false;
	} else {
		@unlink("$backup_path/$backup_filename");
	}

	$data = str_replace("\n", "\n\n", $data);
        $data = explode("\n\n", $data);

	if (count($data) < 10) {
		plugin_configstore_save_debug($device, $debug);
	        return false;
	}

	$lastchange = '';
	$lastuser = '';

	foreach ($data as $d) {
		if (strpos($d, 'Last configuration change at') !== FALSE) {
	        	$lastchange = substr($d, strpos($d, 'change at') + 10, strpos($d, ' by ') - (strpos($d, 'change at') + 10));
	        	$t = explode(' ', $lastchange);
	        	if (isset($t[5])) {
	        		$t = array($t[3], $t[4], $t[5], $t[0], $t[1]);
	           	    	$t = implode(' ', $t);
	                	$lastchange = strtotime($t);

	                	if (substr($d, strpos($d, ' by ')) !== FALSE) {
	                		$lastuser = substr($d, strpos($d, ' by ') + 4);
	                	}
	        	}
		}

		if (substr($d, 0, 9) == 'hostname ') {
			$filename = trim(substr($d, 9));
		        if ($device['hostname'] != $filename) {
		        	$hostname = $filename;
		        }
		}

		if (substr($d, 0, 17) == 'set system name ') {
			$filename = trim(substr($d, 17));
		        if ($device['hostname'] != $filename) {
		        	$hostname = $filename;
		        }
		}

	}

	$backup_results = array();
	$data = implode("\n", $data);
        if (strlen($data) > 100) {
                if (!is_dir("$backup_path/$dir")) {
                        mkdir("$backup_path/$dir", 0777, true);
                }

		$lastbackup = time();
                $date = date("Y-m-d-Hi");
		$filename = "$filename-$date.cfg";
                $file = fopen("$backup_path/$dir/$filename", 'w');
                fwrite($file, $data);
                fclose($file);
                $data = mysql_escape_string($data);

                $t = time();
                if ($lastchange == '') {
                	$lastchange = 0;
		}
                $backup_results = array('device'	=> $device['id'],
					'btime'		=> $t,
					'directory'	=> $dir,
					'filename'	=> $filename,
					'config'	=> $data,
					'lastchange'	=> $lastchange,
					'username'	=> $lastuser,
					'hostname'	=> $hostname,
					);
        } else {
                plugin_configstore_save_debug($device, $debug);
                return false;
        }

	plugin_configstore_save_debug($device, $debug);
	return $backup_results;
}

?>
